import java.util.List;

import org.apache.wicket.Component;
import org.apache.wicket.PageParameters;
import org.apache.wicket.markup.html.WebPage;

/**
 * An interface that allows a component (usually a {@link WebPage} to receive
 * footnote descriptions for footnotes created within this component or it's
 * children.
 * <p/>
 * Creation of footnotes can happen by adding one or more
 * {@link FootnoteBehavior} instances and/or
 * 
 * <pre>
 * &lt;wicket:footnote/&gt;
 * </pre>
 * 
 * tag (see {@link FootnoteResolver} ) and an optional corresponding
 * 
 * <pre>
 * &lt;wicket:footnotedesc/&gt;
 * </pre>
 * 
 * tag (see {@link FootnoteDescriptionResolver}.
 * <p/>
 * By implementing this interface in a consistent manner a proper footnote
 * mechanism can be put in place.
 * 
 * @author Chris
 * 
 */
public interface FootnoteAware {

	/**
	 * Adds the given footnote component to this instance.
	 * 
	 * @param footNoteDesc
	 *            The component containing the footnote description data.
	 * @param addedFromResolver
	 *            Indicates whether the footnote corresponding to the given
	 *            description was created by a behavior or the resolver for
	 *            <code><pre><wicket:footnote/></pre></code> tags.
	 */
	public abstract void addFootnoteDesc(Component footNoteDesc, boolean addedFromResolver);

	/**
	 * If a behavior creates a footnote description component it needs to know
	 * with which <code>id</code> the component has to initialized so it can be
	 * properly added to this <code>FootnoteAware</code> instance's children.
	 * 
	 * @return The <code>id</code> this <code>FootnoteAware</code> instance has
	 *         set for it's footnote description components.
	 */
	public abstract String getFootnoteId();

	/**
	 * A footnote description contains an anchor tag who's name value is defined
	 * externally during creation of the footnote. This method should return a
	 * complete URL that can be used in the footnote to link to it's
	 * description.
	 * <p/>
	 * Usually if this component is a page the result
	 * {@link Component#urlFor(Class, PageParameters)} can be simply prefixed to
	 * the anchor string.
	 * 
	 * @param anchor
	 *            The anchor name including the leading '#'.
	 * @return A complete URL to reference the given anchor in it's page.
	 */
	public abstract String getAnchorLink(String anchor);

	/**
	 * Even though a footnote might appear several times in a page or component
	 * the corresponding description should only be displayed once.
	 * 
	 * @param footNote
	 *            The footnote for which the check should be performed.
	 * @return <code>true</code> if a footnote with the same content has already
	 *         been added to this <code>FootnoteAware</code> instance.
	 */
	public abstract boolean footnoteExists(Component footNote);

	/**
	 * Since footnotes added by a call from {@link FootnoteResolver} need to be
	 * handled differently than those added directly by {@link FootnoteBehavior}
	 * this method is used to retrieve the former ones.
	 * 
	 * @return All footnotes added to this <code>FootnoteAware</code> instance
	 *         by a resolver call.
	 */
	public abstract List<Component> getResolverFootnotes();

	/**
	 * To give component footnotes the same flexibility as resolver footnotes,
	 * e.g. can also be displayed by child components of this instance, this
	 * method allows their retrieval. Those footnotes can be handled in the same
	 * way as resolver footnotes if needed.
	 * 
	 * @return All footnotes added to this <code>FootnoteAware</code> instance
	 *         by a resolver call.
	 */
	public abstract List<Component> getComponentFootnotes();

}
