import java.io.UnsupportedEncodingException;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.wicket.Component;
import org.apache.wicket.Response;
import org.apache.wicket.behavior.AbstractBehavior;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.html.basic.Label;

/**
 * Behavior to allow adding a footnote to a component without having to specify
 * a separate tag in the containing markup. In addition this behavior also adds
 * the corresponding footnote description to the first {@link FootnoteAware}
 * parent in the hierarchy of the component it has been added to if such a
 * parent exists and a description has been specified.
 * <p/>
 * A description will contain the values of the footNote and footNoteText
 * parameters delimited by a colon:
 * <p/>
 * For both, the footnote and it's description, the text specified in the
 * footNote parameter will be wrapped in an anchor tag.
 * 
 * @author Chris
 * 
 */
public class FootnoteBehavior extends AbstractBehavior {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4865561176221628492L;
	private final String footNote;
	private final String footNoteClass;
	private final String footNoteText;
	private final boolean fromResolver;

	private String modelAddtion;

	private static final Logger logger = Logger.getLogger(FootnoteBehavior.class);

	/**
	 * Creates a new behavior instance.
	 * <p/>
	 * This constructor should only be called by {@link FootnoteResolver}
	 * 
	 * @param footNote
	 *            The footnote text to be added to the component's model object
	 *            as superscript and will additionally be wrapped with an anchor
	 *            tag, e.g. 1 would result in <sup><a>1</a></sup>
	 * @param footNoteClass
	 *            If this parameter is not empty it's value will be set as class
	 *            attribute on the generated anchor tag.
	 * @param footNoteText
	 *            If this parameter is not empty a footnote description will be
	 *            created.
	 * @param fromResolver
	 *            Indicates if this behavior was constructed by
	 *            {@link FootnoteResolver}
	 */
	public FootnoteBehavior(String footNote, String footNoteClass, String footNoteText, boolean fromResolver) {
		super();
		this.footNote = StringUtils.trimToEmpty(footNote);
		this.footNoteClass = StringUtils.trimToEmpty(footNoteClass);
		this.footNoteText = StringUtils.trimToEmpty(footNoteText);
		this.fromResolver = fromResolver;
	}

	/**
	 * Creates a new behavior instance.
	 * <p/>
	 * 
	 * @param footNote
	 *            The footnote text to be added to the component's model object
	 *            as superscript and will additionally be wrapped with an anchor
	 *            tag, e.g. 1 would result in <sup><a>1</a></sup>
	 * @param footNoteClass
	 *            If this parameter is not empty it's value will be set as class
	 *            attribute on the generated anchor tag.
	 * @param footNoteText
	 *            If this parameter is not empty a footnote description will be
	 *            created.
	 */
	public FootnoteBehavior(String footNote, String footNoteClass, String footNoteText) {
		this(footNote, footNoteClass, footNoteText, false);

	}

	/**
	 * Creates a new behavior instance.
	 * <p/>
	 * 
	 * @param footNote
	 *            The footnote text to be added to the component's model object
	 *            as superscript and will additionally be wrapped with an anchor
	 *            tag, e.g. 1 would result in <sup><a>1</a></sup>
	 * @param createImageTag
	 *            If set to <code>true</code> the value of <code>footNote</code>
	 *            is wrapped into a html img tag in which it is used as src
	 *            attribute. This allows to use images for a footnote instead of
	 *            regular text.
	 * @param footNoteClass
	 *            If this parameter is not empty it's value will be set as class
	 *            attribute on the generated anchor tag.
	 * @param footNoteText
	 *            If this parameter is not empty a footnote description will be
	 *            created.
	 */
	public FootnoteBehavior(String footNote, boolean createImageTag, String footNoteClass, String footNoteText) {
		this(createImageTag ? "<img src=\"" + footNote + "\"/>" : footNote, footNoteClass, footNoteText);
	}

	/**
	 * Creates a new behavior instance.
	 * <p/>
	 * This constructor should only be called by {@link FootnoteResolver}
	 * 
	 * @param footNote
	 *            The footnote text to be added to the component's model object
	 *            as superscript and will additionally be wrapped with an anchor
	 *            tag, e.g. 1 would result in <sup><a>1</a></sup>
	 * @param createImageTag
	 *            If set to <code>true</code> the value of <code>footNote</code>
	 *            is wrapped into a html img tag in which it is used as src
	 *            attribute. This allows to use images for a footnote instead of
	 *            regular text.
	 * @param footNoteClass
	 *            If this parameter is not empty it's value will be set as class
	 *            attribute on the generated anchor tag.
	 * @param footNoteText
	 *            If this parameter is not empty a footnote description will be
	 *            created.
	 * @param fromResolver
	 *            Indicates if this behavior was constructed by
	 *            {@link FootnoteResolver}
	 */
	public FootnoteBehavior(String footNote, boolean createImageTag, String footNoteClass, String footNoteText,
			boolean fromResolver) {
		this(createImageTag ? "<img src=\"" + footNote + "\"/>" : footNote, footNoteClass, footNoteText, fromResolver);
	}

	/**
	 * Configures the component to display the footnote and if possible to
	 * corresponding {@link FootnoteAware} parent.
	 * 
	 */
	@Override
	public void onConfigure(Component component) {

		String link = null;

		// if no description was given we don't need to add anything to the
		// FootnoteAware parent
		if (StringUtils.isNotEmpty(this.footNoteText)) {
			FootnoteAware fna = component.findParent(FootnoteAware.class);

			// if there is no parent we can't add anything
			if (null != fna) {
				// the anchor value is simply the footNote value converted to
				// hex
				String anchor = this.getHexString(this.footNote);
				// get the complete link required by FootnoteAware
				link = fna.getAnchorLink("#" + anchor);
				// create the footnote description
				Component comp = this.createFootNoteDescription(fna.getFootnoteId(), anchor);
				// add it only if it does not exist yet in the FootnoteAware
				if (!fna.footnoteExists(comp)) {
					fna.addFootnoteDesc(comp, this.fromResolver);
				}
			}
		}
		this.modelAddtion = this.createFootNoteString(link);
	}

	@Override
	public void onRendered(Component component) {
		final Response r = component.getResponse();
		r.write(this.modelAddtion);
	}

	/**
	 * Empty implementation.
	 */
	@Override
	public void beforeRender(Component component) {

	}

	/**
	 * Empty implementation.
	 */
	@Override
	public void bind(Component component) {

	}

	/**
	 * Empty implementation.
	 */
	@Override
	public void detach(Component component) {

	}

	/**
	 * Default implementation returning false.
	 */
	@Override
	public boolean getStatelessHint(Component component) {
		return false;
	}

	/**
	 * Default implementation returning true.
	 */
	@Override
	public boolean isEnabled(Component component) {
		return true;
	}

	/**
	 * Default implementation returning false.
	 */
	@Override
	public boolean isTemporary() {
		return false;
	}

	/**
	 * Empty implementation.
	 */
	@Override
	public void onComponentTag(Component component, ComponentTag tag) {

	}

	/**
	 * Creates the description for a footnote as a label component. The label's
	 * model object contains an anchor tag that wraps the footNote value. If a
	 * footNoteClass was specified it's value will be used as the anchor's class
	 * attribute. Additionally the footNoteText value will be appended to the
	 * label's model object separated by a colon.
	 * 
	 * @param id
	 *            The id to be used for the description component, usually
	 *            retrieved by {@link FootnoteAware#getAnchorLink(String)}.
	 * @param anchor
	 *            The value to be used as the anchor's name value.
	 * @return The label component containing the unescaped html representation
	 *         of a footnote description.
	 */
	private Component createFootNoteDescription(String id, String anchor) {
		StringBuilder builder = new StringBuilder("<a name=\"").append(anchor).append("\"");
		if (StringUtils.isNotEmpty(this.footNoteClass)) {
			builder.append(" class=\"").append(this.footNoteClass).append("\"");
		}
		builder.append(">").append(this.footNote).append("</a>").append(": ").append(this.footNoteText);
		return new Label(id, builder.toString()).setEscapeModelStrings(false);
	}

	/**
	 * Wraps the footNote value into an anchor tag which in turn is wrapped in a
	 * superscript tag. If footNoteClass is not empty it's value is added as
	 * class attribute to the anchor tag. If the link value is not
	 * <code>null</code> it's value will be set as the anchor's href attribute.
	 * 
	 * @param link
	 *            The link to the corresponding description anchor.
	 * @return String representation of the rendered footnote.
	 */
	private String createFootNoteString(String link) {
		StringBuilder builder = new StringBuilder("<sup>");
		String classString = "";
		if (StringUtils.isNotEmpty(this.footNoteClass)) {
			classString = " class=\"" + this.footNoteClass + "\"";
		}

		builder.append("<a");
		if (null != link) {
			builder.append(" href=\"");
			builder.append(link);
			builder.append("\"");
		}
		builder.append(classString);
		builder.append(">");
		builder.append(this.footNote);
		builder.append("</a>");

		builder.append("</sup>");
		return builder.toString();
	}

	/**
	 * Converts text to hex.
	 * 
	 * @param input
	 *            Text value.
	 * @return Hex value.
	 */
	private String getHexString(String input) {

		try {
			return SecurityUtil.convertBytesToHex(input.getBytes("UTF-8"));

		} catch (UnsupportedEncodingException e) {
			logger.error("Could not transform anchor", e);
			return "";
		}

	}

}
