import org.apache.commons.lang.BooleanUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.wicket.Component;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.MarkupStream;
import org.apache.wicket.markup.WicketTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.parser.filter.WicketTagIdentifier;
import org.apache.wicket.markup.resolver.IComponentResolver;

/**
 * Resolver for <wicket:footnotedesc [tagname='xxx'] [fnclass='xxx']
 * [componentfn='xxx']> tags.
 * <p/>
 * <b>Note:</b> Usage of this tag only makes sense in conjunction with at least
 * one <code><wicket:footnote></code> tag or a manually added
 * {@link FootnoteBehavior}. All tags or behaviors (respectively their
 * components) need to be children of the very same {@link FootnoteAware}
 * component.
 * <p/>
 * This resolver concatenates all found footnote descriptions into one label.
 * Every description is wrapped into a tag whose name is taken from the
 * <code>tagname</code> attribute. If no <code>tagname</code> is given the
 * default value <code>span</code> is used.
 * <p/>
 * If a <code>fnclass</code> attribute is given it's value is used as the
 * wrapping tag's class attribute.
 * <p/>
 * If componentfn contains 'yes', 'on' or 'true' (case insensitive) the
 * footnotes added by components will be used otherwise the default (footnotes
 * added by a resolver) will be used.
 * <p/>
 * 
 * @author Chris
 * 
 */
public class FootnoteDescriptionResolver implements IComponentResolver {

	/**
	 * 
	 */
	private static final long serialVersionUID = -1477352695878654256L;

	private static final String tagname = "footnotedesc";

	private static final String defaultTag = "span";

	// register the tagname
	static {
		WicketTagIdentifier.registerWellKnownTagName(tagname);
	}

	public boolean resolve(MarkupContainer container, MarkupStream markupStream, ComponentTag ctag) {

		if ((ctag instanceof WicketTag) && tagname.equalsIgnoreCase(ctag.getName())) {
			WicketTag wtag = (WicketTag) ctag;

			String clazz = StringUtils.trimToNull(wtag.getAttribute("fnclass"));
			String tag = StringUtils.trimToNull(wtag.getAttribute("tagname"));
			boolean useComponentFns = BooleanUtils.toBoolean(wtag.getAttribute("componentfn"));

			if (null == tag) {
				tag = defaultTag;
			}

			FootnoteAware fna = null;

			if (FootnoteAware.class.isInstance(container)) {
				fna = (FootnoteAware) container;
			}

			if (null == fna) {
				fna = container.findParent(FootnoteAware.class);
			}

			if (null != fna) {
				StringBuilder builder = new StringBuilder();
				for (Component comp : (useComponentFns ? fna.getComponentFootnotes() : fna.getResolverFootnotes())) {
					builder.append("<").append(tag);
					if (null != clazz) {
						builder.append(" class=\"");
						builder.append(clazz);
						builder.append("\"");
					}
					builder.append(">").append(comp.getDefaultModelObjectAsString()).append("</").append(tag)
							.append(">");
				}
				Label label = new Label("_" + tag + "_" + container.getPage().getAutoIndex(), builder.toString());
				label.setRenderBodyOnly(container.getApplication().getMarkupSettings().getStripWicketTags());
				label.setEscapeModelStrings(false);
				container.autoAdd(label, markupStream);

			}
			return true;

		}
		return false;
	}
}
