import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.wicket.Application;
import org.apache.wicket.Component;
import org.apache.wicket.MarkupContainer;
import org.apache.wicket.Response;
import org.apache.wicket.WicketRuntimeException;
import org.apache.wicket.markup.ComponentTag;
import org.apache.wicket.markup.MarkupElement;
import org.apache.wicket.markup.MarkupException;
import org.apache.wicket.markup.MarkupStream;
import org.apache.wicket.markup.WicketTag;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.parser.filter.WicketTagIdentifier;
import org.apache.wicket.markup.resolver.IComponentResolver;
import org.apache.wicket.response.StringResponse;
import org.apache.wicket.util.lang.PropertyResolver;
import org.apache.wicket.util.string.Strings;
import org.apache.wicket.util.string.interpolator.MapVariableInterpolator;

/**
 * Resolver for <code><wicket:footnote index='xxx'|image='xxx' key='xxx' [fnclass='xxx'] [desc='xxx']></code> tags.
 * <p/>
 * For each tag a {@link Label} component with an associated
 * {@link FootnoteBehavior} is created. As that behavior is created via a
 * resolver the corresponding footnote description is created accordingly which
 * means in order to display them a <wicket:footnotedesc> tag (see
 * {@link FootnoteDescriptionResolver})must be used in a markup who's component
 * is the child of the same {@link FootnoteAware} instance as the component
 * who's markup contains this <wicket:footnote> tag.
 * <p/>
 * <b>Attribute description</b>
 * <table border="1" style="text-align:center;">
 * <tr>
 * <th>Attribute</th>
 * <th>Mandatory</th>
 * <th>Description</th>
 * </tr>
 * <tr>
 * <td>key</td>
 * <td>Yes</td>
 * <td>Value will be looked up as a property using the container's
 * {@link MarkupContainer#getString(String)} method.</td>
 * </tr>
 * <tr>
 * <td>index</td>
 * <td>Yes - if image is empty, must be empty otherwise</td>
 * <td>If set it's value will be looked up as a property using the container's
 * {@link MarkupContainer#getString(String)} method used as the footnote.</td>
 * </tr>
 * <tr>
 * <td>image</td>
 * <td>Yes - if index is empty, must be empty otherwise</td>
 * <td>If set it's value will be looked up as a property using the container's
 * {@link MarkupContainer#getString(String)} method and used as src attribute of
 * an image tag which in turn will be used as footnote.</td>
 * </tr>
 * <tr>
 * <td>fnclass</td>
 * <td>No</td>
 * <td>If set it's value will be used as class attribute of the generated anchor
 * tags.</td>
 * </tr>
 * <tr>
 * <td>desc</td>
 * <td>No</td>
 * <td>If set it's value will be looked up as a property using the container's
 * {@link MarkupContainer#getString(String)} method and a footnote description
 * will be created containing the footnote and the description value separated
 * by a colon.</td>
 * </tr>
 * </table>
 * <p/>
 * All of those attributes are subject to variable interpolation. Variable
 * values can be specified by the containing component's properties or by nested
 * tags.
 * <p/>
 * <b>Example:</b>
 * <p/>
 * Markup:
 * 
 * <pre>
 * &lt;wicket:footnote key=&quot;footnote.label&quot; index=&quot;${index}&quot; class=&quot;someClass&quot; desc=&quot;Explanation&quot;&gt;
 *   &lt;span wicket:id=&quot;index&quot;&gt;&lt;/span&gt;
 *   &lt;span wicket:id=&quot;name&quot;&gt;&lt;/span&gt;
 * &lt;/wicket:footnote&gt;
 * </pre>
 * 
 * Java code:
 * 
 * <pre>
 * public SomeComponent(){
 *   ...
 *   this.add(new Label(&quot;index&quot;,&quot;1&quot;));
 *   this.add(new Label(&quot;name&quot;,&quot;Footnote&quot;));
 *   ...
 * }
 * </pre>
 * 
 * Property file:
 * 
 * <pre>
 * footnote.label=${name} Label
 * </pre>
 * 
 * Result:
 * <p/>
 * <i>Footnote:</i>
 * 
 * <pre>
 *   Footnote Label&lt;sup&gt;&lt;a href=&quot;...#31&quot; class=&quot;someClass&quot;&gt;1&lt;/a&gt;&lt;/sup&gt;
 * </pre>
 * 
 * <i>Footnote Description:</i>
 * 
 * <pre>
 *   &lt;sup&gt;&lt;a name=&quot;31&quot; class=&quot;someClass&quot;&gt;1&lt;/a&gt;&lt;/sup&gt;: Explanation
 * </pre>
 * 
 * <b>Note:</b> The footnote description is not used as is but instead get's
 * further wrapped by the resolver for the <wicket:footnotedesc> tag, see
 * {@link FootnoteDescriptionResolver}.
 * <p/>
 * 
 * @author Chris
 * 
 */
public class FootnoteResolver implements IComponentResolver {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1438893433802866476L;

	private static final String tagname = "footnote";

	private static final Logger logger = Logger.getLogger(FootnoteResolver.class);

	// register the tagname
	static {
		WicketTagIdentifier.registerWellKnownTagName(tagname);
	}

	/**
	 * Resolves the footnote according to the given attributes in the
	 * <code><wicket:footnote/></code> tag. The key attribute will be looked up
	 * as a property while the other attributes are used directly. The key
	 * attribute is mandatory and exactly one of index or image attribute has to
	 * be specified. Internally a {@link FootnoteBehavior} will be created
	 * according to the attribute values.
	 * <p/>
	 * It is also possible to use interpolation to substitute variables in all
	 * attributes by using nested tags or properties of the footnote's parent.
	 * 
	 */
	public boolean resolve(MarkupContainer container, MarkupStream markupStream, ComponentTag tag) {

		if ((tag instanceof WicketTag) && tagname.equalsIgnoreCase(tag.getName())) {
			WicketTag wtag = (WicketTag) tag;

			String imagePath = StringUtils.trimToNull(wtag.getAttribute("image"));
			String key = StringUtils.trimToNull(wtag.getAttribute("key"));
			String index = StringUtils.trimToNull(wtag.getAttribute("index"));
			String desc = StringUtils.trimToNull(wtag.getAttribute("desc"));
			String clazz = StringUtils.trimToNull(wtag.getAttribute("fnclass"));

			String msg = null;

			if (null == key) {
				throw new MarkupException(
						"Wrong format of <wicket:footnote index='xxx'|image='xxx' key='xxx' [fnclass='xxx'] [desc='xxx']>: specify the key attribute");
			}

			msg = StringUtils.trimToNull(container.getString(key));
			if (null == msg) {
				msg = key;
			}

			boolean isImage = true;

			if (null != index) {
				if (null != imagePath) {
					throw new MarkupException(
							"Wrong format of <wicket:footnote index='xxx'|image='xxx' key='xxx' [fnclass='xxx'] [desc='xxx']>: you must not use image and index attribtue at once");
				}
				isImage = false;
				String tmp = container.getString(index);

				if (null != tmp) {
					index = tmp;
				}

			} else if (null == imagePath) {
				throw new MarkupException(
						"Wrong format of <wicket:footnote index='xxx'|image='xxx' key='xxx' [fnclass='xxx'] [desc='xxx']>: specify the image or index attribute");
			} else {
				index = container.getString(imagePath);

				if (null == index) {
					index = imagePath;
				}
			}

			if (null != desc) {
				String tmp = container.getString(desc);
				if (null != tmp) {
					desc = tmp;
				}
			}

			final String id = "_" + tagname + "_" + container.getPage().getAutoIndex();
			FootnoteContainer ic = new FootnoteContainer(id, msg, index, desc, clazz, markupStream, container);

			// container.autoAdd(ic, markupStream);

			ic.onComponentTagBody(markupStream, tag);

			Label label = new Label(id, ic.msg);
			FootnoteBehavior fnb = new FootnoteBehavior(ic.index, isImage, ic.clazz, ic.desc, true);
			label.add(fnb);

			label.setRenderBodyOnly(container.getApplication().getMarkupSettings().getStripWicketTags());
			container.autoAdd(label, markupStream);
			return true;

		}
		return false;
	}

	/**
	 * Container class to render the footnote.
	 * 
	 * @author Chris
	 * 
	 */
	private static final class FootnoteContainer extends MarkupContainer {

		/**
		 * 
		 */
		private static final long serialVersionUID = -8318528409385831166L;
		private String msg;
		private String index;
		private String desc;
		private String clazz;
		private final MarkupContainer parent;

		private final Set<String> replacedChildren = new HashSet<String>();

		public FootnoteContainer(String id, String msg, String index, String desc, String clazz,
				MarkupStream markupStream, MarkupContainer parent) {
			super(id);
			this.msg = msg;
			this.index = index;
			this.desc = desc;
			this.clazz = clazz;
			this.parent = parent;
		}

		/**
		 * Wrapper to logger this instance's settings.
		 * 
		 * @return String containing the instance's settings.
		 */
		private final String logParams(String param) {
			return "<wicket:footnote> tag with param = '" + param + "'";
		}

		/**
		 * Renders the component tag. For each member all contained variables
		 * are replaced by the following methods in this order:
		 * <ul>
		 * <li>trying to get the value from a child component that has the
		 * variable's value as id</li>
		 * <li>trying to get the value as property from the parent's model
		 * object</li>
		 * <li>trying to get the value as property from the parent object</li>
		 * </ul>
		 * 
		 * If all attempts to lookup up a variable fail or not all child tags
		 * are replaced either an exception is thrown or a warning is logged
		 * depending on the resource settings.
		 * 
		 */
		@Override
		protected void onComponentTagBody(final MarkupStream markupStream, final ComponentTag openTag) {

			// get all child tags' rendered contents
			Map<String, String> children = this.getRenderedChildren(markupStream, openTag);

			this.msg = this.doInterpolate(this.msg, children, markupStream);
			this.index = this.doInterpolate(this.index, children, markupStream);
			if (null != this.desc) {
				this.desc = this.doInterpolate(this.desc, children, markupStream);
			}
			if (null != this.clazz) {
				this.clazz = this.doInterpolate(this.clazz, children, markupStream);
			}
			// Make sure all of the children were rendered
			Iterator<String> iter = children.keySet().iterator();
			while (iter.hasNext()) {
				String id = iter.next();
				if (this.replacedChildren.contains(id) == false) {
					String msg = "The <wicket:footnote> for index " + this.index
							+ "has a child element with wicket:id=\"" + id + "\". You must add the variable ${" + id
							+ "} to the file content for the wicket:include.";

					if (this.isThrowingExceptions() == true) {
						markupStream.throwMarkupException(msg);
					} else {
						logger.warn(msg);
					}
				}
			}
		}

		/**
		 * Helper method to interpolate the variables in given a String.
		 * 
		 * @param param
		 * @param children
		 * @param markupStream
		 * @return
		 */
		private String doInterpolate(final String param, Map<String, String> children, final MarkupStream markupStream) {

			// substitute all variables and set the result as response
			return new MapVariableInterpolator(param, children) {

				@Override
				protected String getValue(String variableName) {

					// try to get value from rendered child tags
					String value = super.getValue(variableName);

					if (null != value) {
						// if we used a child tag create a marker
						FootnoteContainer.this.replacedChildren.add(variableName);
					} else {
						if (logger.isDebugEnabled()) {
							logger.debug(FootnoteContainer.this.logParams(param)
									+ " - Could not find replacement value for variable = '" + variableName
									+ "' in child tags");
						}
						// see if the variable can be replaced using the
						// parent's model object - use try/catch to avoid fail
						// early
						try {
							value = Strings.toString(PropertyResolver.getValue(variableName, FootnoteContainer.this
									.getParent().getDefaultModelObject()));
						} catch (WicketRuntimeException e) {
							if (logger.isDebugEnabled()) {
								logger.debug(FootnoteContainer.this.logParams(param)
										+ " - Could not find replacement value for variable = '" + variableName
										+ "' in parent model");
							}
						}
					}

					if (null == value) {
						// see if the variable can be replaced using the parent
						// object - use try/catch to avoid fail early
						try {
							value = Strings.toString(PropertyResolver.getValue(variableName, FootnoteContainer.this
									.getParent()));
						} catch (WicketRuntimeException e) {
							if (logger.isDebugEnabled()) {
								logger.debug(FootnoteContainer.this.logParams(param)
										+ " - Could not find replacement value for variable = '" + variableName
										+ "' in parent object");
							}
						}
					}

					if (null == value) {
						// Handle failed lookup
						String logMsg = FootnoteContainer.this.logParams(param)
								+ " - Bailing with exception since variable = '" + variableName
								+ "' could not be replaced";

						if (FootnoteContainer.this.isThrowingExceptions()) {

							if (logger.isDebugEnabled()) {
								logger.debug(logMsg);

							}
							markupStream.throwMarkupException(FootnoteContainer.this.logParams(param)
									+ " - Could not replace variable  = '" + variableName + "'");

						} else {
							logger.warn(logMsg);
						}
					}

					return value;
				}

			}.toString();

		}

		/**
		 * Wraps call to resource settings to determine if a exception should be
		 * thrown in case not all variables are resolved or not all children are
		 * rendered.
		 * 
		 * @return
		 *         <code>Application.get().getResourceSettings().getThrowExceptionOnMissingResource()</code>
		 */
		private final boolean isThrowingExceptions() {
			return Application.get().getResourceSettings().getThrowExceptionOnMissingResource();
		}

		/**
		 * Walks through all children of the current tag and stores their
		 * rendering results in a map.
		 * 
		 * @param markupStream
		 *            The markupStream associated with the tag.
		 * @param openTag
		 *            The current include tag to be rendered.
		 * @return Map containing all children's rendered responses associated
		 *         with each child's id.
		 */
		private Map<String, String> getRenderedChildren(final MarkupStream markupStream, final ComponentTag openTag) {
			Map<String, String> children = new HashMap<String, String>();

			if (!openTag.isOpenClose()) {

				while (markupStream.hasMore() && !markupStream.get().closes(openTag)) {
					MarkupElement element = markupStream.get();
					// If it a tag like <wicket..> or <span wicket:id="..." >
					if ((element instanceof ComponentTag) && !markupStream.atCloseTag()) {
						String id = ((ComponentTag) element).getId();

						Component comp = this.parent.get(id);
						if (comp != null) {
							children.put(id, this.getRenderedResponseString(markupStream, comp));
						} else {
							markupStream.next();
						}
					} else {
						markupStream.next();
					}
				}

			}

			return children;

		}

		/**
		 * Obtains the rendered response of the given component rendered
		 * according to the markupStream. Rendering uses a temporary response
		 * and the original response is restored before returning the result.
		 * Only the body of each component is rendered.
		 * 
		 * @param markupStream
		 *            The markupStream used to render the component.
		 * @param comp
		 *            The component to be rendered.
		 * @return The rendered result.
		 */
		private String getRenderedResponseString(final MarkupStream markupStream, Component comp) {
			Response webResponse = comp.getResponse();

			StringResponse response = new StringResponse();
			boolean oldValue = comp.getRenderBodyOnly();
			try {
				comp.setRenderBodyOnly(true);
				this.getRequestCycle().setResponse(response);

				comp.render(markupStream);
			} finally {
				this.getRequestCycle().setResponse(webResponse);
				comp.setRenderBodyOnly(oldValue);
			}
			return response.getBuffer().toString();

		}

		/**
		 * 
		 * @see org.apache.wicket.MarkupContainer#isTransparentResolver()
		 */
		@Override
		public boolean isTransparentResolver() {
			return true;
		}

		public String getMsg() {
			return this.msg;
		}

		public String getIndex() {
			return this.index;
		}

		public String getDesc() {
			return this.desc;
		}
	}
}
